package eu.digitalconcepts.dcgw.client;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URISyntaxException;
import java.net.URL;
import java.text.SimpleDateFormat;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;

import eu.digitalconcepts.dcgw.api.Message;
import eu.digitalconcepts.dcgw.api.device.DeviceMessage;
import eu.digitalconcepts.dcgw.api.device.DevicesMessage;
import eu.digitalconcepts.dcgw.api.telegram.StatesMessage;
import eu.digitalconcepts.dcgw.api.telegram.TelegramMessage;
import eu.digitalconcepts.dcgw.client.util.Base64;



/**
 * Example to process the streaming API.
 * 
 * 
 * @author  Digital Concepts GmbH, T. Hennerich
 */
public class StreamingExample
{
	// ===================================================================================================================================
	// Field Variables
	// ===================================================================================================================================

	/** JSON converter. */
	private final ObjectMapper objectMapper = new ObjectMapper();
	
	
	// ===================================================================================================================================
	// Main
	// ===================================================================================================================================

	public static void main(String[] pArgs) throws IOException, URISyntaxException
	{
		String url      = "http://localhost:8080/devices/stream?delimited=lengthCharacters";
		String login    = "admin";
		String password = "admin";
		
		if(pArgs!=null && pArgs.length>0)
		{
			if(pArgs.length!=3)
				throw new IllegalArgumentException("invalid number of parameters: URL login password");
			
			url      = pArgs[0];
			login    = pArgs[1];
			password = pArgs[2];
		}
		
		new StreamingExample(new URL(url), login, password);
	}
	
	
	// ===================================================================================================================================
	// Constructor
	// ===================================================================================================================================

	public StreamingExample(URL pUrl, String pLogin, String pPassword) throws IOException
	{
        // Prepare JSON converter
        objectMapper.setSerializationInclusion(JsonInclude.Include.NON_NULL);
		objectMapper.enable(SerializationFeature.INDENT_OUTPUT);
		objectMapper.setDateFormat(new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSZ"));
		objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);

		HttpURLConnection connection = null;

		try
		{
			// Connect to gateway
			connection = (HttpURLConnection)pUrl.openConnection();
			connection.setRequestProperty("Authorization", "Basic " + Base64.encode(pLogin + ":" + pPassword));
			
			int responseCode = connection.getResponseCode();
 
            if(responseCode!=200)
            	throw new IOException("connection failure, http status " + responseCode);
            
            log("connected to " + pUrl + ": got " + responseCode);
            
    		// Read character stream with delimited=lengthCharacters
            BufferedReader reader = new BufferedReader(new InputStreamReader(connection.getInputStream(), "UTF-8"));
            
            while(true)
            {
            	int    length    = Integer.valueOf(reader.readLine());
            	char[] jsonChars = new char[length];
            	int	   read		 = 0;
            	
            	while(read<length)
            		read += reader.read(jsonChars, read, length-read);

            	// Convert to general message for content inspection
            	String  json    = String.valueOf(jsonChars);
            	Message message = objectMapper.readValue(json, Message.class);
            	String  content = message.getHeader().getContent();
            	
            	// Handle individual content
            	if(content.equals("states"))
            		handleStates(objectMapper.readValue(json, StatesMessage.class));
            	else if(content.equals("device"))
            		handleDevice(objectMapper.readValue(json, DeviceMessage.class));
            	else if(content.equals("devices"))
            		handleDevices(objectMapper.readValue(json, DevicesMessage.class));
            	else if(content.equals("telegram"))
            		handleTelegram(objectMapper.readValue(json, TelegramMessage.class));
            	else
            		log("unhandled content " + content);
            }
		}
		finally
		{
			if(connection!=null)
				connection.disconnect();
		}
	}
	
	private void handleStates(StatesMessage pStatesMessage) throws JsonProcessingException
	{
		log("\nhandle " + pStatesMessage.getHeader().getContent());
		log(objectMapper.writeValueAsString(pStatesMessage));
	}
	
	private void handleDevice(DeviceMessage pDeviceMessage) throws JsonProcessingException
	{
		log("\nhandle " + pDeviceMessage.getHeader().getContent());
		log(objectMapper.writeValueAsString(pDeviceMessage));
	}
	
	private void handleDevices(DevicesMessage pDevicesMessage) throws JsonProcessingException
	{
		log("\nhandle " + pDevicesMessage.getHeader().getContent());
		log(objectMapper.writeValueAsString(pDevicesMessage));
	}

	
	private void handleTelegram(TelegramMessage pTelegramMessage) throws JsonProcessingException
	{
		log("\nhandle " + pTelegramMessage.getHeader().getContent());
		log(objectMapper.writeValueAsString(pTelegramMessage));
	}
	
	private void log(String pLog)
	{
		System.out.println(pLog);
	}
}
